﻿using System;
using System.IO;
using System.Text;
using System.Collections.Generic;
using System.Runtime.InteropServices;

namespace FoxitPrintManager
{
    public class PrintManagerDLL
    {
        private class DLL
        {
            public IntPtr dllHandle = IntPtr.Zero;
            private List<int> shutDownList = new List<int>();
            public bool loadError = false;

            public void RegisterForShutdown(int managerID)
            {
                shutDownList.Add(managerID);
            }

            private IntPtr GetProc(string functionName)
            {
                IntPtr result = GetProcAddress(dllHandle, functionName);
                if (result == IntPtr.Zero)
                {
                    loadError = true;
                }
                return result;
            }

            public DLL(string dllFileName)
            {
                dllHandle = LoadLibraryEx(dllFileName, IntPtr.Zero, 0x1100);
                if (dllHandle != IntPtr.Zero)
                {
                    loadError = false;
                    NewPrintManager = (DNewPrintManager)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_NewPrintManager"), typeof(DNewPrintManager));
                    GetPrintManagerErrorCode = (DGetPrintManagerErrorCode)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_GetPrintManagerErrorCode"), typeof(DGetPrintManagerErrorCode));
                    InitPrintManager = (DInitPrintManager)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_InitPrintManager"), typeof(DInitPrintManager));

                    GetPrinterCount = (DGetPrinterCount)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_GetPrinterCount"), typeof(DGetPrinterCount));
                    GetPrinterName = (DGetPrinterName)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_GetPrinterName"), typeof(DGetPrinterName));
                    GetPaperSizeCount = (DGetPaperSizeCount)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_GetPaperSizeCount"), typeof(DGetPaperSizeCount));
                    GetPaperSizeWidth = (DGetPaperSizeWidth)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_GetPaperSizeWidth"), typeof(DGetPaperSizeWidth));
                    GetPaperSizeHeight = (DGetPaperSizeHeight)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_GetPaperSizeHeight"), typeof(DGetPaperSizeHeight));
                    GetPaperSizeName = (DGetPaperSizeName)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_GetPaperSizeName"), typeof(DGetPaperSizeName));
                    GetPaperSizeID = (DGetPaperSizeID)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_GetPaperSizeID"), typeof(DGetPaperSizeID));
                    GetPaperSourceCount = (DGetPaperSourceCount)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_GetPaperSourceCount"), typeof(DGetPaperSourceCount));
                    GetPaperSourceName = (DGetPaperSourceName)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_GetPaperSourceName"), typeof(DGetPaperSourceName));
                    GetPaperSourceID = (DGetPaperSourceID)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_GetPaperSourceID"), typeof(DGetPaperSourceID));
                    GetDefaultPrinterName = (DGetDefaultPrinterName)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_GetDefaultPrinterName"), typeof(DGetDefaultPrinterName));
                    GetDefaultPaperSize = (DGetDefaultPaperSize)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_GetDefaultPaperSize"), typeof(DGetDefaultPaperSize));
                    GetDefaultPaperSource = (DGetDefaultPaperSource)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_GetDefaultPaperSource"), typeof(DGetDefaultPaperSource));

                    PrintPDFFromFile = (DPrintPDFFromFile)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_PrintPDFFromFile"), typeof(DPrintPDFFromFile));
                    PrintPDFFromMemory = (DPrintPDFFromMemory)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_PrintPDFFromMemory"), typeof(DPrintPDFFromMemory));
                    PrintPDFFromFileToPrinter = (DPrintPDFFromFileToPrinter)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_PrintPDFFromFileToPrinter"), typeof(DPrintPDFFromFileToPrinter));
                    PrintPDFFromMemoryToPrinter = (DPrintPDFFromMemoryToPrinter)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_PrintPDFFromMemoryToPrinter"), typeof(DPrintPDFFromMemoryToPrinter));

                    NewPrintJob = (DNewPrintJob)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_NewPrintJob"), typeof(DNewPrintJob));
                    AddPDFFromFileToJob = (DAddPDFFromFileToJob)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_AddPDFFromFileToJob"), typeof(DAddPDFFromFileToJob));
                    AddPDFFromMemoryToJob = (DAddPDFFromMemoryToJob)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_AddPDFFromMemoryToJob"), typeof(DAddPDFFromMemoryToJob));
                    BeginPrintJob = (DBeginPrintJob)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_BeginPrintJob"), typeof(DBeginPrintJob));
                    CancelPrintJob = (DCancelPrintJob)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_CancelPrintJob"), typeof(DCancelPrintJob));
                    GetPrintJobStatus = (DGetPrintJobStatus)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_GetPrintJobStatus"), typeof(DGetPrintJobStatus));
                    ClosePrintJob = (DClosePrintJob)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_ClosePrintJob"), typeof(DClosePrintJob));

                    SetJobCopies = (DSetJobCopies)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_SetJobCopies"), typeof(DSetJobCopies));
                    SetJobColor = (DSetJobColor)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_SetJobColor"), typeof(DSetJobColor));
                    SetJobDuplex = (DSetJobDuplex)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_SetJobDuplex"), typeof(DSetJobDuplex));
                    SetJobRotation = (DSetJobRotation)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_SetJobRotation"), typeof(DSetJobRotation));
                    SetJobScaling = (DSetJobScaling)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_SetJobScaling"), typeof(DSetJobScaling));
                    SetJobCustomScale = (DSetJobCustomScale)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_SetJobCustomScale"), typeof(DSetJobCustomScale));
                    SetJobPaperSource = (DSetJobPaperSource)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_SetJobPaperSource"), typeof(DSetJobPaperSource));
                    SetJobPaperSize = (DSetJobPaperSize)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_SetJobPaperSize"), typeof(DSetJobPaperSize));
                    SetJobPageList = (DSetJobPageList)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_SetJobPageList"), typeof(DSetJobPageList));
                    SetJobPriority = (DSetJobPriority)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_SetJobPriority"), typeof(DSetJobPriority));
                    SetJobQuality = (DSetJobQuality)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_SetJobQuality"), typeof(DSetJobQuality));
                    SetJobDocumentName = (DSetJobDocumentName)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_SetJobDocumentName"), typeof(DSetJobDocumentName));
                    SetJobCollate = (DSetJobCollate)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_SetJobCollate"), typeof(DSetJobCollate));
                    SetJobPrinter = (DSetJobPrinter)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_SetJobPrinter"), typeof(DSetJobPrinter));

                    GetJobCopies = (DGetJobCopies)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_GetJobCopies"), typeof(DGetJobCopies));
                    GetJobColor = (DGetJobColor)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_GetJobColor"), typeof(DGetJobColor));
                    GetJobDuplex = (DGetJobDuplex)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_GetJobDuplex"), typeof(DGetJobDuplex));
                    GetJobRotation = (DGetJobRotation)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_GetJobRotation"), typeof(DGetJobRotation));
                    GetJobScaling = (DGetJobScaling)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_GetJobScaling"), typeof(DGetJobScaling));
                    GetJobCustomScale = (DGetJobCustomScale)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_GetJobCustomScale"), typeof(DGetJobCustomScale));
                    GetJobPaperSource = (DGetJobPaperSource)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_GetJobPaperSource"), typeof(DGetJobPaperSource));
                    GetJobPaperSize = (DGetJobPaperSize)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_GetJobPaperSize"), typeof(DGetJobPaperSize));
                    GetJobPageListCount = (DGetJobPageListCount)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_GetJobPageListCount"), typeof(DGetJobPageListCount));
                    GetJobPageListItem = (DGetJobPageListItem)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_GetJobPageListItem"), typeof(DGetJobPageListItem));
                    GetJobPriority = (DGetJobPriority)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_GetJobPriority"), typeof(DGetJobPriority));
                    GetJobQuality = (DGetJobQuality)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_GetJobQuality"), typeof(DGetJobQuality));
                    GetJobDocumentName = (DGetJobDocumentName)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_GetJobDocumentName"), typeof(DGetJobDocumentName));
                    GetJobCollate = (DGetJobCollate)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_GetJobCollate"), typeof(DGetJobCollate));
                    GetJobPrinter = (DGetJobPrinter)Marshal.GetDelegateForFunctionPointer(GetProc("FXPM_GetJobPrinter"), typeof(DGetJobPrinter));

                    if (loadError)
                    {
                        FreeLibrary(dllHandle);
                        dllHandle = IntPtr.Zero;
                    }
                }
            }

            ~DLL()
            {
                Release();
            }

            public void Release()
            {
                if (dllHandle != IntPtr.Zero)
                {
                    foreach (int managerID in shutDownList)
                    {
                        // call manager free
                    }
                    FreeLibrary(dllHandle);
                    dllHandle = IntPtr.Zero;
                }
            }

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DNewPrintManager();

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DGetPrintManagerErrorCode(int managerID);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DInitPrintManager(int managerID, [MarshalAs(UnmanagedType.LPUTF8Str)] string sn, [MarshalAs(UnmanagedType.LPUTF8Str)] string key);


            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DGetPrinterCount(int managerID);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate IntPtr DGetPrinterName(int managerID, int printerIndex);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DGetPaperSizeCount(int managerID, [MarshalAs(UnmanagedType.LPUTF8Str)] string printerName);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DGetPaperSizeWidth(int managerID, [MarshalAs(UnmanagedType.LPUTF8Str)] string printerName, int paperIndex);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DGetPaperSizeHeight(int managerID, [MarshalAs(UnmanagedType.LPUTF8Str)] string printerName, int paperIndex);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate IntPtr DGetPaperSizeName(int managerID, [MarshalAs(UnmanagedType.LPUTF8Str)] string printerName, int paperIndex);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DGetPaperSizeID(int managerID, [MarshalAs(UnmanagedType.LPUTF8Str)] string printerName, int paperIndex);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DGetPaperSourceCount(int managerID, [MarshalAs(UnmanagedType.LPUTF8Str)] string printerName);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate IntPtr DGetPaperSourceName(int managerID, [MarshalAs(UnmanagedType.LPUTF8Str)] string printerName, int trayIndex);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DGetPaperSourceID(int managerID, [MarshalAs(UnmanagedType.LPUTF8Str)] string printerName, int trayIndex);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate IntPtr DGetDefaultPrinterName(int managerID);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DGetDefaultPaperSize(int managerID, [MarshalAs(UnmanagedType.LPUTF8Str)] string printerName);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DGetDefaultPaperSource(int managerID, [MarshalAs(UnmanagedType.LPUTF8Str)] string printerName);


            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DPrintPDFFromFile(int managerID, [MarshalAs(UnmanagedType.LPUTF8Str)] string pdfFileName, [MarshalAs(UnmanagedType.LPUTF8Str)] string pdfPassword);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DPrintPDFFromMemory(int managerID, IntPtr pdfData, Int64 pdfDataLength, [MarshalAs(UnmanagedType.LPUTF8Str)] string pdfPassword);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DPrintPDFFromFileToPrinter(int managerID, [MarshalAs(UnmanagedType.LPUTF8Str)] string pdfFileName, [MarshalAs(UnmanagedType.LPUTF8Str)] string pdfPassword, [MarshalAs(UnmanagedType.LPUTF8Str)] string printerName);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DPrintPDFFromMemoryToPrinter(int managerID, IntPtr pdfData, Int64 pdfDataLength, [MarshalAs(UnmanagedType.LPUTF8Str)] string pdfPassword, [MarshalAs(UnmanagedType.LPUTF8Str)] string printerName);


            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DNewPrintJob(int managerID);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DAddPDFFromFileToJob(int managerID, int jobID, [MarshalAs(UnmanagedType.LPUTF8Str)] string pdfFileName, [MarshalAs(UnmanagedType.LPUTF8Str)] string pdfPassword);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DAddPDFFromMemoryToJob(int managerID, int jobID, IntPtr pdfData, Int64 pdfDataLength, [MarshalAs(UnmanagedType.LPUTF8Str)] string pdfPassword);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DBeginPrintJob(int managerID, int jobID);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DCancelPrintJob(int managerID, int jobID);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DGetPrintJobStatus(int managerID, int jobID);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DClosePrintJob(int managerID, int jobID);


            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DSetJobCopies(int managerID, int jobID, int newCopies);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DSetJobColor(int managerID, int jobID, int newColorMode);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DSetJobDuplex(int managerID, int jobID, int newDuplex);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DSetJobRotation(int managerID, int jobID, int newRotation);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DSetJobScaling(int managerID, int jobID, int newScaling);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DSetJobCustomScale(int managerID, int jobID, double newCustomScalePercent);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DSetJobPaperSource(int managerID, int jobID, int newPaperSourceID);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DSetJobPaperSize(int managerID, int jobID, int newPaperID);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DSetJobPageList(int managerID, int jobID, int documentIndex, [MarshalAs(UnmanagedType.LPUTF8Str)] string pageList);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DSetJobPriority(int managerID, int jobID, int newPriority);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DSetJobQuality(int managerID, int jobID, int newQuality);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DSetJobDocumentName(int managerID, int jobID, [MarshalAs(UnmanagedType.LPUTF8Str)] string newDocumentName);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DSetJobCollate(int managerID, int jobID, int newCollate);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DSetJobPrinter(int managerID, int jobID, [MarshalAs(UnmanagedType.LPUTF8Str)] string printerName);


            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DGetJobCopies(int managerID, int jobID);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DGetJobColor(int managerID, int jobID);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DGetJobDuplex(int managerID, int jobID);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DGetJobRotation(int managerID, int jobID);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DGetJobScaling(int managerID, int jobID);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate double DGetJobCustomScale(int managerID, int jobID);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DGetJobPaperSource(int managerID, int jobID);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DGetJobPaperSize(int managerID, int jobID);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DGetJobPageListCount(int managerID, int jobID, int documentIndex);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DGetJobPageListItem(int managerID, int jobID, int documentIndex, int itemIndex);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DGetJobPriority(int managerID, int jobID);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DGetJobQuality(int managerID, int jobID);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate IntPtr DGetJobDocumentName(int managerID, int jobID);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate int DGetJobCollate(int managerID, int jobID);

            [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
            internal delegate IntPtr DGetJobPrinter(int managerID, int jobID);

            public DNewPrintManager NewPrintManager;
            public DGetPrintManagerErrorCode GetPrintManagerErrorCode;
            public DInitPrintManager InitPrintManager;

            public DGetPrinterCount GetPrinterCount;
            public DGetPrinterName GetPrinterName;
            public DGetPaperSizeCount GetPaperSizeCount;
            public DGetPaperSizeWidth GetPaperSizeWidth;
            public DGetPaperSizeHeight GetPaperSizeHeight;
            public DGetPaperSizeName GetPaperSizeName;
            public DGetPaperSizeID GetPaperSizeID;
            public DGetPaperSourceCount GetPaperSourceCount;
            public DGetPaperSourceName GetPaperSourceName;
            public DGetPaperSourceID GetPaperSourceID;
            public DGetDefaultPrinterName GetDefaultPrinterName;
            public DGetDefaultPaperSize GetDefaultPaperSize;
            public DGetDefaultPaperSource GetDefaultPaperSource;

            public DPrintPDFFromFile PrintPDFFromFile;
            public DPrintPDFFromMemory PrintPDFFromMemory;
            public DPrintPDFFromFileToPrinter PrintPDFFromFileToPrinter;
            public DPrintPDFFromMemoryToPrinter PrintPDFFromMemoryToPrinter;

            public DNewPrintJob NewPrintJob;
            public DAddPDFFromFileToJob AddPDFFromFileToJob;
            public DAddPDFFromMemoryToJob AddPDFFromMemoryToJob;
            public DBeginPrintJob BeginPrintJob;
            public DCancelPrintJob CancelPrintJob;
            public DGetPrintJobStatus GetPrintJobStatus;
            public DClosePrintJob ClosePrintJob;

            public DSetJobCopies SetJobCopies;
            public DSetJobColor SetJobColor;
            public DSetJobDuplex SetJobDuplex;
            public DSetJobRotation SetJobRotation;
            public DSetJobScaling SetJobScaling;
            public DSetJobCustomScale SetJobCustomScale;
            public DSetJobPaperSource SetJobPaperSource;
            public DSetJobPaperSize SetJobPaperSize;
            public DSetJobPageList SetJobPageList;
            public DSetJobPriority SetJobPriority;
            public DSetJobQuality SetJobQuality;
            public DSetJobDocumentName SetJobDocumentName;
            public DSetJobCollate SetJobCollate;
            public DSetJobPrinter SetJobPrinter;

            public DGetJobCopies GetJobCopies;
            public DGetJobColor GetJobColor;
            public DGetJobDuplex GetJobDuplex;
            public DGetJobRotation GetJobRotation;
            public DGetJobScaling GetJobScaling;
            public DGetJobCustomScale GetJobCustomScale;
            public DGetJobPaperSource GetJobPaperSource;
            public DGetJobPaperSize GetJobPaperSize;
            public DGetJobPageListCount GetJobPageListCount;
            public DGetJobPageListItem GetJobPageListItem;
            public DGetJobPriority GetJobPriority;
            public DGetJobQuality GetJobQuality;
            public DGetJobDocumentName GetJobDocumentName;
            public DGetJobCollate GetJobCollate;
            public DGetJobPrinter GetJobPrinter;

            public string UTF8IntPtrToString(IntPtr utf8)
            {
                int len = MultiByteToWideChar(65001, 0, utf8, -1, null, 0);
                if (len == 0) throw new System.ComponentModel.Win32Exception();
                var buf = new StringBuilder(len);
                len = MultiByteToWideChar(65001, 0, utf8, -1, buf, len);
                return buf.ToString();
            }

            [DllImport("kernel32.dll", SetLastError = true)]
            static extern IntPtr LoadLibraryEx(string lpFileName, IntPtr hReservedNull, uint dwFlags);

            [DllImport("kernel32.dll")]
            internal static extern int FreeLibrary(IntPtr hModule);

            [DllImport("kernel32.dll")]
            internal static extern IntPtr GetProcAddress(IntPtr hModule, String procname);

            [DllImport("kernel32.dll", CharSet = CharSet.Auto, SetLastError = true)]
            private static extern int MultiByteToWideChar(int codepage, int flags, IntPtr utf8, int utf8len, StringBuilder buffer, int buflen);
        }

        private DLL dll = null;
        private int managerID = 0;

        public PrintManagerDLL(string printManagerDLLFileName)
        {
            string absolutePath = Path.GetFullPath(printManagerDLLFileName);
            dll = new DLL(absolutePath);
            if (dll.dllHandle != IntPtr.Zero)
            {
                managerID = dll.NewPrintManager();
                dll.RegisterForShutdown(managerID);
            }
            else
            {
                dll = null;
            }
        }

        public bool LibraryLoaded()
        {
            return dll != null;
        }

        public void ReleaseLibrary()
        {
            if (dll != null) dll.Release();
            dll = null;
        }

        public int GetPrintManagerErrorCode()
        {
            if (dll == null) return 0;
            else
            {
                return dll.GetPrintManagerErrorCode(managerID);
            }
        }

        public int InitPrintManager(string sn, string key)
        {
            if (dll == null) return 0;
            else
                return dll.InitPrintManager(managerID, sn, key);
        }


        public int GetPrinterCount()
        {
            if (dll == null) return 0;
            else
                return dll.GetPrinterCount(managerID);
        }

        public string GetPrinterName(int printerIndex)
        {
            if (dll == null) return "";
            else
            {
                return dll.UTF8IntPtrToString(dll.GetPrinterName(managerID, printerIndex));
            }
        }

        public int GetPaperSizeCount(string printerName)
        {
            if (dll == null) return 0;
            else
                return dll.GetPaperSizeCount(managerID, printerName);
        }

        public int GetPaperSizeWidth(string printerName, int paperIndex)
        {
            if (dll == null) return 0;
            else
                return dll.GetPaperSizeWidth(managerID, printerName, paperIndex);
        }

        public int GetPaperSizeHeight(string printerName, int paperIndex)
        {
            if (dll == null) return 0;
            else
                return dll.GetPaperSizeHeight(managerID, printerName, paperIndex);
        }

        public string GetPaperSizeName(string printerName, int paperIndex)
        {
            if (dll == null) return "";
            else
            {
                return dll.UTF8IntPtrToString(dll.GetPaperSizeName(managerID, printerName, paperIndex));
            }
        }

        public int GetPaperSizeID(string printerName, int paperIndex)
        {
            if (dll == null) return 0;
            else
                return dll.GetPaperSizeID(managerID, printerName, paperIndex);
        }

        public int GetPaperSourceCount(string printerName)
        {
            if (dll == null) return 0;
            else
                return dll.GetPaperSourceCount(managerID, printerName);
        }

        public string GetPaperSourceName(string printerName, int trayIndex)
        {
            if (dll == null) return "";
            else
            {
                return dll.UTF8IntPtrToString(dll.GetPaperSourceName(managerID, printerName, trayIndex));
            }
        }

        public int GetPaperSourceID(string printerName, int trayIndex)
        {
            if (dll == null) return 0;
            else
            {
                return dll.GetPaperSourceID(managerID, printerName, trayIndex);
            }
        }

        public string GetDefaultPrinterName()
        {
            if (dll == null) return "";
            else
            {
                return dll.UTF8IntPtrToString(dll.GetDefaultPrinterName(managerID));
            }
        }

        public int GetDefaultPaperSize(string printerName)
        {
            if (dll == null) return 0;
            else
            {
                return dll.GetDefaultPaperSize(managerID, printerName);
            }
        }

        public int GetDefaultPaperSource(string printerName)
        {
            if (dll == null) return 0;
            else
            {
                return dll.GetDefaultPaperSource(managerID, printerName);
            }
        }


        public int PrintPDFFromFile(string pdfFileName, string pdfPassword)
        {
            if (dll == null) return 0;
            else
                return dll.PrintPDFFromFile(managerID, pdfFileName, pdfPassword);
        }

        public int PrintPDFFromMemory(byte[] pdfData, string pdfPassword)
        {
            if (dll == null) return 0;
            else
            {
                GCHandle gch = GCHandle.Alloc(pdfData, GCHandleType.Pinned);
                int result = dll.PrintPDFFromMemory(managerID, gch.AddrOfPinnedObject(), pdfData.Length, pdfPassword);
                gch.Free();
                return result;
            }
        }

        public int PrintPDFFromFileToPrinter(string pdfFileName, string pdfPassword, string printerName)
        {
            if (dll == null) return 0;
            else
                return dll.PrintPDFFromFileToPrinter(managerID, pdfFileName, pdfPassword, printerName);
        }

        public int PrintPDFFromMemoryToPrinter(byte[] pdfData, string pdfPassword, string printerName)
        {
            if (dll == null) return 0;
            else
            {
                GCHandle gch = GCHandle.Alloc(pdfData, GCHandleType.Pinned);
                int result = dll.PrintPDFFromMemoryToPrinter(managerID, gch.AddrOfPinnedObject(), pdfData.Length, pdfPassword, printerName);
                gch.Free();
                return result;
            }
        }


        public int NewPrintJob()
        {
            if (dll == null) return 0;
            else
                return dll.NewPrintJob(managerID);
        }

        public int AddPDFFromFileToJob(int jobID, string pdfFileName, string pdfPassword)
        {
            if (dll == null) return 0;
            else
                return dll.AddPDFFromFileToJob(managerID, jobID, pdfFileName, pdfPassword);
        }

        public int AddPDFFromMemoryToJob(int jobID, byte[] pdfData, string pdfPassword)
        {
            if (dll == null) return 0;
            else
            {
                GCHandle gch = GCHandle.Alloc(pdfData, GCHandleType.Pinned);
                int result = dll.AddPDFFromMemoryToJob(managerID, jobID, gch.AddrOfPinnedObject(), pdfData.Length, pdfPassword);
                gch.Free();
                return result;
            }
        }

        public int BeginPrintJob(int jobID)
        {
            if (dll == null) return 0;
            else
                return dll.BeginPrintJob(managerID, jobID);
        }

        public int CancelPrintJob(int jobID)
        {
            if (dll == null) return 0;
            else
                return dll.CancelPrintJob(managerID, jobID);
        }

        public int GetPrintJobStatus(int jobID)
        {
            if (dll == null) return 0;
            else
                return dll.GetPrintJobStatus(managerID, jobID);
        }

        public int ClosePrintJob(int jobID)
        {
            if (dll == null) return 0;
            else
                return dll.CancelPrintJob(managerID, jobID);
        }


        public int SetJobCopies(int jobID, int newCopies)
        {
            if (dll == null) return 0;
            else
                return dll.SetJobCopies(managerID, jobID, newCopies);
        }

        public int SetJobColor(int jobID, int newColorMode)
        {
            if (dll == null) return 0;
            else
                return dll.SetJobColor(managerID, jobID, newColorMode);
        }

        public int SetJobDuplex(int jobID, int newDuplex)
        {
            if (dll == null) return 0;
            else
                return dll.SetJobDuplex(managerID, jobID, newDuplex);
        }

        public int SetJobRotation(int jobID, int newRotation)
        {
            if (dll == null) return 0;
            else
                return dll.SetJobRotation(managerID, jobID, newRotation);
        }

        public int SetJobScaling(int jobID, int newScaling)
        {
            if (dll == null) return 0;
            else
                return dll.SetJobScaling(managerID, jobID, newScaling);
        }

        public int SetJobCustomScale(int jobID, double newCustomScalePercent)
        {
            if (dll == null) return 0;
            else
                return dll.SetJobCustomScale(managerID, jobID, newCustomScalePercent);
        }

        public int SetJobPaperSource(int jobID, int newPaperSourceID)
        {
            if (dll == null) return 0;
            else
                return dll.SetJobPaperSource(managerID, jobID, newPaperSourceID);
        }

        public int SetJobPaperSize(int jobID, int newPaperID)
        {
            if (dll == null) return 0;
            else
                return dll.SetJobPaperSize(managerID, jobID, newPaperID);
        }

        public int SetJobPageList(int jobID, int documentIndex, string pageList)
        {
            if (dll == null) return 0;
            else
                return dll.SetJobPageList(managerID, jobID, documentIndex, pageList);
        }

        public int SetJobPriority(int jobID, int newPriority)
        {
            if (dll == null) return 0;
            else
                return dll.SetJobPriority(managerID, jobID, newPriority);
        }

        public int SetJobQuality(int jobID, int newQuality)
        {
            if (dll == null) return 0;
            else
                return dll.SetJobQuality(managerID, jobID, newQuality);
        }

        public int SetJobDocumentName(int jobID, string newDocumentName)
        {
            if (dll == null) return 0;
            else
                return dll.SetJobDocumentName(managerID, jobID, newDocumentName);
        }

        public int SetJobCollate(int jobID, int newCollate)
        {
            if (dll == null) return 0;
            else
                return dll.SetJobCollate(managerID, jobID, newCollate);
        }

        public int SetJobPrinter(int jobID, string newPrinter)
        {
            if (dll == null) return 0;
            else
                return dll.SetJobPrinter(managerID, jobID, newPrinter);
        }

        public int GetJobCopies(int jobID)
        {
            if (dll == null) return 0;
            else
            {
                return dll.GetJobCopies(managerID, jobID);
            }
        }

        public int GetJobColor(int jobID)
        {
            if (dll == null) return 0;
            else
            {
                return dll.GetJobColor(managerID, jobID);
            }
        }

        public int GetJobDuplex(int jobID)
        {
            if (dll == null) return 0;
            else
            {
                return dll.GetJobDuplex(managerID, jobID);
            }
        }

        public int GetJobRotation(int jobID)
        {
            if (dll == null) return 0;
            else
            {
                return dll.GetJobRotation(managerID, jobID);
            }
        }

        public int GetJobScaling(int jobID)
        {
            if (dll == null) return 0;
            else
            {
                return dll.GetJobCopies(managerID, jobID);
            }
        }

        public double GetJobCustomScale(int jobID)
        {
            if (dll == null) return 0;
            else
            {
                return dll.GetJobCustomScale(managerID, jobID);
            }
        }

        public int GetJobPaperSource(int jobID)
        {
            if (dll == null) return 0;
            else
            {
                return dll.GetJobPaperSource(managerID, jobID);
            }
        }

        public int GetJobPaperSize(int jobID)
        {
            if (dll == null) return 0;
            else
            {
                return dll.GetJobPaperSize(managerID, jobID);
            }
        }

        public int GetJobPageListCount(int jobID, int documentIndex)
        {
            if (dll == null) return 0;
            else
            {
                return dll.GetJobPageListCount(managerID, jobID, documentIndex);
            }
        }


        public int GetJobPageListItem(int jobID, int documentIndex, int itemIndex)
        {
            if (dll == null) return 0;
            else
            {
                return dll.GetJobPageListItem(managerID, jobID, documentIndex, itemIndex);
            }
        }

        public int GetJobPriority(int jobID)
        {
            if (dll == null) return 0;
            else
            {
                return dll.GetJobPriority(managerID, jobID);
            }
        }

        public int GetJobQuality(int jobID)
        {
            if (dll == null) return 0;
            else
            {
                return dll.GetJobQuality(managerID, jobID);
            }
        }

        public string GetJobDocumentName(int jobID)
        {
            if (dll == null) return "";
            else
            {
                return dll.UTF8IntPtrToString(dll.GetJobDocumentName(managerID, jobID));
            }
        }

        public int GetJobCollate(int jobID)
        {
            if (dll == null) return 0;
            else
            {
                return dll.GetJobCollate(managerID, jobID);
            }
        }

        public string GetJobPrinter(int jobID)
        {
            if (dll == null) return "";
            else
            {
                return dll.UTF8IntPtrToString(dll.GetJobPrinter(managerID, jobID));
            }
        }
    }
}
